/*
 * Copyright DataStax, Inc.
 *
 * This software can be used solely with DataStax Enterprise. Please consult the license at
 * http://www.datastax.com/terms/datastax-dse-driver-license-terms
 */
package com.datastax.dse.driver.api.core.metadata;

import com.datastax.dse.driver.api.core.metadata.schema.DseKeyspaceMetadata;
import com.datastax.oss.driver.api.core.CqlIdentifier;
import com.datastax.oss.driver.api.core.metadata.Metadata;
import edu.umd.cs.findbugs.annotations.NonNull;
import java.util.Map;
import java.util.Optional;

/**
 * The metadata of the DSE cluster that this driver instance is connected to.
 *
 * <p>It specializes the open-source driver's metadata to use custom schema classes, that may have
 * additional fields.
 */
public interface DseMetadata extends Metadata {

  @NonNull
  @Override
  Map<CqlIdentifier, ? extends DseKeyspaceMetadata> getKeyspaces();

  @NonNull
  @Override
  default Optional<? extends DseKeyspaceMetadata> getKeyspace(@NonNull CqlIdentifier keyspaceId) {
    return Optional.ofNullable(getKeyspaces().get(keyspaceId));
  }

  @NonNull
  @Override
  default Optional<? extends DseKeyspaceMetadata> getKeyspace(@NonNull String keyspaceName) {
    return getKeyspace(CqlIdentifier.fromCql(keyspaceName));
  }
}
