/*
 * Copyright DataStax, Inc.
 *
 * This software can be used solely with DataStax Enterprise. Please consult the license at
 * http://www.datastax.com/terms/datastax-dse-driver-license-terms
 */
package com.datastax.dse.driver.api.core.metadata.schema;

import com.datastax.oss.driver.api.core.CqlIdentifier;
import com.datastax.oss.driver.api.core.metadata.schema.ClusteringOrder;
import com.datastax.oss.driver.api.core.metadata.schema.RelationMetadata;
import com.datastax.oss.driver.shaded.guava.common.collect.ImmutableList;
import edu.umd.cs.findbugs.annotations.NonNull;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * Specialized table or materialized view metadata for the DSE driver.
 *
 * <p>The only difference with {@link RelationMetadata} is that all columns are represented as
 * {@link DseColumnMetadata} instances.
 */
public interface DseRelationMetadata extends RelationMetadata {

  @NonNull
  @Override
  default List<? extends DseColumnMetadata> getPrimaryKey() {
    return ImmutableList.<DseColumnMetadata>builder()
        .addAll(getPartitionKey())
        .addAll(getClusteringColumns().keySet())
        .build();
  }

  @NonNull
  @Override
  List<? extends DseColumnMetadata> getPartitionKey();

  @NonNull
  @Override
  Map<? extends DseColumnMetadata, ClusteringOrder> getClusteringColumns();

  @NonNull
  @Override
  Map<CqlIdentifier, ? extends DseColumnMetadata> getColumns();

  @NonNull
  @Override
  default Optional<? extends DseColumnMetadata> getColumn(@NonNull CqlIdentifier columnId) {
    return Optional.ofNullable(getColumns().get(columnId));
  }

  @NonNull
  @Override
  default Optional<? extends DseColumnMetadata> getColumn(@NonNull String columnName) {
    return getColumn(CqlIdentifier.fromCql(columnName));
  }
}
