/*
 * Copyright DataStax, Inc.
 *
 * This software can be used solely with DataStax Enterprise. Please consult the license at
 * http://www.datastax.com/terms/datastax-dse-driver-license-terms
 */
package com.datastax.dse.driver.internal.core.session;

import com.datastax.dse.driver.api.core.DseSession;
import com.datastax.dse.driver.api.core.metadata.DseMetadata;
import com.datastax.oss.driver.api.core.session.Session;
import com.datastax.oss.driver.internal.core.session.SessionWrapper;
import edu.umd.cs.findbugs.annotations.NonNull;
import edu.umd.cs.findbugs.annotations.Nullable;
import java.util.concurrent.CompletionStage;
import net.jcip.annotations.ThreadSafe;

/**
 * Implementation note: metadata methods perform unchecked casts, relying on the fact that the
 * metadata manager returns the appropriate runtime type.
 */
@ThreadSafe
public class DefaultDseSession extends SessionWrapper implements DseSession {

  public DefaultDseSession(Session delegate) {
    super(delegate);
  }

  @NonNull
  @Override
  public DseMetadata getMetadata() {
    return (DseMetadata) getDelegate().getMetadata();
  }

  @NonNull
  @Override
  @SuppressWarnings("unchecked")
  public CompletionStage<? extends DseMetadata> setSchemaMetadataEnabled(
      @Nullable Boolean newValue) {
    return (CompletionStage<? extends DseMetadata>)
        getDelegate().setSchemaMetadataEnabled(newValue);
  }

  @NonNull
  @Override
  public DseMetadata refreshSchema() {
    return (DseMetadata) getDelegate().refreshSchema();
  }

  @NonNull
  @Override
  @SuppressWarnings("unchecked")
  public CompletionStage<? extends DseMetadata> refreshSchemaAsync() {
    return (CompletionStage<? extends DseMetadata>) getDelegate().refreshSchemaAsync();
  }
}
