## Metadata

The DSE driver exposes a specialized [DseMetadata], that extends the
[OSS driver's metadata][OSS metadata] to add DSE-specific information:

```java
DseSession session = DseSession.builder().build();
DseMetadata metadata = session.getMetadata();
```

### Nodes

`DseMetadata.getNodes()` returns regular `Node` instances, but they hold additional key/value pairs
in their `getExtras()` field. The keys are listed in [DseNodeProperties]; refer to the javadocs of
that class for more details, in particular the runtime type of the values.

For example, here's how to retrieve the DSE workloads of a node:

```java
Node node = metadata.getNodes().values().iterator().next();
Set<String> workloads = (Set<String>) node.getExtras().get(DseNodeProperties.DSE_WORKLOADS);
```

### Schema metadata

`DseMetadata.getKeyspaces()` return a collection of [DseKeyspaceMetadata] instances. All nested
schema elements are also DSE-specific extensions:

```java
DseKeyspaceMetadata ks = metadata.getKeyspace("ks").get();
DseTableMetadata users = ks.getTable("users").get();
DseColumnMetadata id = users.getColumn("id").get();
```

Currently (DSE 6.0), the only types that really add extra information are:

* [DseFunctionMetadata]: add support for the `DETERMINISTIC` and `MONOTONIC` keywords; 
* [DseAggregateMetadata]: add support for the `MONOTONIC` keyword.

All other types (keyspaces, tables, etc.) are virtually identical to their OSS counterparts. They
only exist to facilitate future extensibility.


[DseMetadata]:          https://docs.datastax.com/en/drivers/java-dse/2.0/com/datastax/dse/driver/api/core/metadata/DseMetadata.html
[DseNodeProperties]:    https://docs.datastax.com/en/drivers/java-dse/2.0/com/datastax/dse/driver/api/core/metadata/DseNodeProperties.html
[DseKeyspaceMetadata]:  https://docs.datastax.com/en/drivers/java-dse/2.0/com/datastax/dse/driver/api/core/metadata/schema/DseKeyspaceMetadata.html
[DseFunctionMetadata]:  https://docs.datastax.com/en/drivers/java-dse/2.0/com/datastax/dse/driver/api/core/metadata/schema/DseFunctionMetadata.html
[DseAggregateMetadata]: https://docs.datastax.com/en/drivers/java-dse/2.0/com/datastax/dse/driver/api/core/metadata/schema/DseAggregateMetadata.html

[OSS metadata]: https://docs.datastax.com/en/developer/java-driver/4.0-beta/manual/core/metadata/